/*
   Copyright 2009 Last.fm Ltd. 
      - Primarily authored by Max Howell, Jono Cole and Doug Mansell

   This file is part of liblastfm.

   liblastfm is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   liblastfm is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with liblastfm.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef LASTFM_PLAYLIST_H
#define LASTFM_PLAYLIST_H

#include <lastfm/global.h>
#include <lastfm/Xspf>
#include <QString>
#include <QUrl>

namespace lastfm
{
    class LASTFM_DLLEXPORT Playlist
    {
        int m_id;
    
        Playlist() : m_id( -1 )
        {}

    public:
        Playlist( int id ) : m_id( id )
        {}
    
        int id() const { return m_id; }

        QNetworkReply* addTrack( const Track& ) const;
        QNetworkReply* fetch() const;

        static QNetworkReply* create( const QString& title, const QString& description = "" );
        static QNetworkReply* fetch( const QUrl& url );
    
        static Xspf fetch( QNetworkReply* );
    };
}

#endif
